# TP 19 (fichier à compléter) : Graphes
# BCPST1B 2024-2025
# Lycée Hoche, Versailles
# L.-C. LEFÈVRE

#%% démarrage (exécuter une fois pour toutes)

# La bibliothèque NetworkX permet de manipuler des graphes, et notamment (pour nous) de les dessiner.
# On les crée à partir de nx.DiGraph() (graphe orienté) ou nx.Graph() (non-orienté) depuis une matrice d'adjacente (qui doit être un tableau numpy) ou un dictionnaire d'adjacence.
# Documentation pour dessiner :
# https://networkx.org/documentation/stable/reference/generated/networkx.drawing.nx_pylab.draw_networkx.html
import networkx as nx
import matplotlib.pyplot as plt
import numpy as np

# dessine un graphe orienté donné par matrice d'adjacence (vous pouvez jouer avec les couleurs…)
def dessiner(M):
    MM = np.array(M)
    G = nx.DiGraph(MM)
    nx.draw_networkx(G, arrows=True, arrowsize=20, node_size=500, node_color="powderblue", font_size=12, linewidths=1, width=1, edgecolors="black")
    plt.box(False)
    plt.show()

#%% exemple partie II.1 : une matrice d'adjacence

M = [[1, 1, 1, 0], [1, 0, 1, 0], [0, 0, 0, 1], [0, 1, 1, 0]]
dessiner(M)

#%% exemple partie II.1 : carte de France

trains = {"Paris": ["Versailles", "Lyon", "Luxembourg"], "Versailles": ["Paris"], "Lyon": ["Paris", "Marseille", "Grenoble", "Orange"], "Grenoble": ["Lyon"], "Marseille": ["Lyon", "Istres", "Orange"], "Istres": ["Marseille"], "Orange": ["Lyon", "Marseille"], "Luxembourg": ["Paris"]}

positions = {"Paris": (0, 0), "Versailles": (-0.2, -0.2), "Lyon": (0.3, -0.4), "Grenoble": (0.5, -0.7), "Marseille": (0.38, -1), "Istre": (0.15, -0.9), "Orange": (0.3, -0.7), "Luxembourg": (0.5, 0.25)}

G = nx.DiGraph(trains)
nx.draw_networkx(G, arrows=False, node_size=2000, font_size=8, font_color="blue", font_weight="bold", node_color="pink", linewidths=1, edgecolors="black", pos=positions)
plt.box(False)
plt.show()

# Votre ville n'est pas dans la liste ? Ajoutez-là, avec sa position !

#%% exercice 1.1

def liste_vers_matrice(L):
    ...

L = [[0, 1, 2], [0, 2], [3], [1, 2]]
print(liste_vers_matrice(L))

#%% exercice 1.2

def matrice_vers_liste(M):
    ...

M = [[1, 1, 1, 0], [1, 0, 1, 0], [0, 0, 0, 1], [0, 1, 1, 0]]
print(matrice_vers_liste(M))

#%% exercice 2

def graphe_complet(n):
    ...

M = graphe_complet(5)
dessiner(M)

#%% exercice 3

def graphe_cyclique(n):
    ...

M = graphe_cyclique(6)
dessiner(M)

#%% exercice 4

def graphe_biparti_complet(n, p):
    ...

M = graphe_biparti_complet(3, 4)
dessiner(M)

#%% exercice 5.1

def degré_sortant(M, x):
    ...

# exemple : celui de la section II.1
M = [[1, 1, 1, 0], [1, 0, 1, 0], [0, 0, 0, 1], [0, 1, 1, 0]]
for x in range(len(M)):
    print("Sommet", x, "degré sortant", degré_sortant(M, x))

#%% exercice 5.2

def degré_entrant(M, x):
    ...

M = [[1, 1, 1, 0], [1, 0, 1, 0], [0, 0, 0, 1], [0, 1, 1, 0]]
for x in range(len(M)):
    print("Sommet", x, "degré entrant", degré_entrant(M, x))

#%% exercice 5.3

def degré(M, x):
    ...

M = [[1, 1, 1, 0], [1, 0, 1, 0], [0, 0, 0, 1], [0, 1, 1, 0]]
for x in range(len(M)):
    print("Sommet", x, "degré", degré(M, x))

#%% exercice 5.4

def degré_total(M):
    ...

M = [[1, 1, 1, 0], [1, 0, 1, 0], [0, 0, 0, 1], [0, 1, 1, 0]]
print(degré_total(M))

#%% exercice 8

def est_chemin_possible(M, C):
    ...

M = [[1, 1, 1, 0], [1, 0, 1, 0], [0, 0, 0, 1], [0, 1, 1, 0]]
C = [0, 1, 2, 3, 1, 2]
print(est_chemin_possible(M, C))
D = [0, 1, 2, 0, 1]
print(est_chemin_possible(M, D))

#%% exercice 9

def longueur(M, C):
    ...

# test : tableau des distances entre villes de France
# 0 Paris
# 1 Lyon
# 2 Nice
# 3 Nantes
# 4 Strasbourg
# 5 Montpellier
# 6 Lille
# 7 Rennes
# 8 Reims
# 9 Saint-Étienne
# 10 Angers
# 11 Grenoble
# 12 Nîmes
# 13 Aix-en-Provence
# 14 Brest
M = [[0, 462, 931, 380, 488, 746, 219, 348, 143, 522, 293, 575, 710, 759, 589],
     [462, 0, 472, 681, 494, 302, 689, 718, 487, 62, 594, 107, 251, 299, 970],
     [930, 471, 0, 1143, 790, 326, 1157, 1186, 955, 490, 1062, 465, 279, 176, 1440],
     [380, 682, 1145, 0, 860, 824, 597, 107, 515, 662, 88, 786, 874, 972, 296],
     [487, 491, 788, 860, 0, 791, 549, 827, 347, 550, 773, 533, 739, 787, 1069],
     [746, 303, 327, 823, 791, 0, 963, 895, 787, 322, 771, 297, 56, 154, 1120],
     [217, 690, 1159, 598, 522, 963, 0, 572, 199, 749, 511, 803, 938, 986, 759],
     [347, 718, 1186, 106, 827, 894, 572, 0, 483, 699, 126, 823, 948, 1013, 241],
     [143, 487, 955, 516, 347, 786, 198, 483, 0, 546, 429, 600, 735, 783, 725],
     [523, 64, 491, 661, 552, 322, 750, 698, 548, 0, 574, 155, 271, 319, 950],
     [293, 595, 1062, 88, 773, 770, 511, 128, 429, 576, 0, 700, 824, 890, 377],
     [574, 106, 466, 786, 534, 296, 801, 823, 599, 155, 699, 0, 245, 293, 1075],
     [711, 252, 281, 872, 740, 55, 938, 949, 736, 271, 825, 246, 0, 108, 1169],
     [757, 298, 178, 970, 786, 153, 984, 1013, 781, 317, 889, 291, 106, 0, 1267],
     [590, 971, 1441, 298, 1070, 1121, 760, 241, 725, 952, 378, 1076, 1171, 1269, 0]]
# Paris Lyon Grenoble Nice Strasbourg Nantes Paris
C = [0, 1, 11, 2, 4, 3, 0]
print(longueur(M, C))

#%% exercice 10

...
